local MER, E, L, V, P, G = unpack(select(2, ...))
local MERB = E:GetModule("mUIBags")

local MERB = _Bags

local bankBags = {BANK_CONTAINER}
local MAX_MOVE_TIME = 1.25
for i = NUM_BAG_SLOTS + 1, NUM_BAG_SLOTS + NUM_BANKBAGSLOTS do
	table.insert(bankBags, i)
end

local playerBags = {}
for i = 0, NUM_BAG_SLOTS do
	table.insert(playerBags, i)
end

local allBags = {}
for _,i in ipairs(playerBags) do
	table.insert(allBags, i)
end
for _,i in ipairs(bankBags) do
	table.insert(allBags, i)
end

local coreGroups = {
	guild = guildBags,
	bank = bankBags,
	bags = playerBags,
	all = allBags,
}

local bagCache = {}
local bagIDs = {}
local bagQualities = {}
local bagPetIDs = {}
local bagStacks = {}
local bagMaxStacks = {}
local bagGroups = {}
local initialOrder = {}
local bagSorted, bagLocked = {}, {}
local bagRole
local moves = {}
local targetItems = {}
local sourceUsed = {}
local targetSlots = {}
local specialtyBags = {}
local emptySlots = {}

local moveRetries = 0
local movesUnderway, lastItemID, lockStop, lastDestination, lastMove
local moveTracker = {}

local inventorySlots = {
	INVTYPE_AMMO = 0,
	INVTYPE_HEAD = 1,
	INVTYPE_NECK = 2,
	INVTYPE_SHOULDER = 3,
	INVTYPE_BODY = 4,
	INVTYPE_CHEST = 5,
	INVTYPE_ROBE = 5,
	INVTYPE_WAIST = 6,
	INVTYPE_LEGS = 7,
	INVTYPE_FEET = 8,
	INVTYPE_WRIST = 9,
	INVTYPE_HAND = 10,
	INVTYPE_FINGER = 11,
	INVTYPE_TRINKET = 12,
	INVTYPE_CLOAK = 13,
	INVTYPE_WEAPON = 14,
	INVTYPE_SHIELD = 15,
	INVTYPE_2HWEAPON = 16,
	INVTYPE_WEAPONMAINHAND = 18,
	INVTYPE_WEAPONOFFHAND = 19,
	INVTYPE_HOLDABLE = 20,
	INVTYPE_RANGED = 21,
	INVTYPE_THROWN = 22,
	INVTYPE_RANGEDRIGHT = 23,
	INVTYPE_RELIC = 24,
	INVTYPE_TABARD = 25,
}

local safe = {
	[BANK_CONTAINER] = true,
	[0] = true
}

local frame = CreateFrame("Frame")
local t, WAIT_TIME = 0, 0.05
frame:SetScript("OnUpdate", function(self, elapsed)
	t = t + (elapsed or 0.01)
	if t > WAIT_TIME then
		t = 0
		MERB:DoMoves()
	end
end)
frame:Hide()
MERB.SortUpdateTimer = frame

local function IsGuildBankBag(bagid)
	return (bagid > 50 and bagid <= 58)
end

local function UpdateLocation(from, to)
	if (bagIDs[from] == bagIDs[to]) and (bagStacks[to] < bagMaxStacks[to]) then
		local stackSize = bagMaxStacks[to]
		if (bagStacks[to] + bagStacks[from]) > stackSize then
			bagStacks[from] = bagStacks[from] - (stackSize - bagStacks[to])
			bagStacks[to] = stackSize
		else
			bagStacks[to] = bagStacks[to] + bagStacks[from]
			bagStacks[from] = nil
			bagIDs[from] = nil
			bagQualities[from] = nil
			bagMaxStacks[from] = nil
		end
	else
		bagIDs[from], bagIDs[to] = bagIDs[to], bagIDs[from]
		bagQualities[from], bagQualities[to] = bagQualities[to], bagQualities[from]
		bagStacks[from], bagStacks[to] = bagStacks[to], bagStacks[from]
		bagMaxStacks[from], bagMaxStacks[to] = bagMaxStacks[to], bagMaxStacks[from]
	end
end

local function PrimarySort(a, b)
	local aName, _, _, aLvl, _, _, _, _, _, _, aPrice = GetItemInfo(bagIDs[a])
	local bName, _, _, bLvl, _, _, _, _, _, _, bPrice = GetItemInfo(bagIDs[b])

	if aLvl ~= bLvl and aLvl and bLvl then
		return aLvl > bLvl
	end
	if aPrice ~= bPrice and aPrice and bPrice then
		return aPrice > bPrice
	end

	if aName and bName then
		return aName < bName
	end
end

local function DefaultSort(a, b)
	local aID = bagIDs[a]
	local bID = bagIDs[b]

	if (not aID) or (not bID) then return aID end

	if bagPetIDs[a] and bagPetIDs[b] then
		local aName, _, aType = C_PetJournal.GetPetInfoBySpeciesID(aID);
		local bName, _, bType = C_PetJournal.GetPetInfoBySpeciesID(bID);

		if aType and bType and aType ~= bType then
			return aType > bType
		end

		if aName and bName and aName ~= bName then
			return aName < bName
		end
	end

	local aOrder, bOrder = initialOrder[a], initialOrder[b]

	if aID == bID then
		local aCount = bagStacks[a]
		local bCount = bagStacks[b]
		if aCount and bCount and aCount == bCount then
			return aOrder < bOrder
		elseif aCount and bCount then
			return aCount < bCount
		end
	end

	local _, _, aRarity, _, _, _, _, _, aEquipLoc, _, _, aItemClassId, aItemSubClassId = GetItemInfo(aID)
	local _, _, bRarity, _, _, _, _, _, bEquipLoc, _, _, bItemClassId, bItemSubClassId = GetItemInfo(bID)

	aRarity = bagQualities[a]
	bRarity = bagQualities[b]

	if bagPetIDs[a] then
		aRarity = 1
	end

	if bagPetIDs[b] then
		bRarity = 1
	end

	if aRarity ~= bRarity and aRarity and bRarity then
		return aRarity > bRarity
	end

	if aItemClassId ~= bItemClassId then
		return (aItemClassId or 99) < (bItemClassId or 99)
	end

	if aItemClassId == LE_ITEM_CLASS_ARMOR or aItemClassId == LE_ITEM_CLASS_WEAPON then
		local aEquipLoc = inventorySlots[aEquipLoc] or -1
		local bEquipLoc = inventorySlots[bEquipLoc] or -1
		if aEquipLoc == bEquipLoc then
			return PrimarySort(a, b)
		end

		if aEquipLoc and bEquipLoc then
			return aEquipLoc < bEquipLoc
		end
	end
	if (aItemClassId == bItemClassId) and (aItemSubClassId == bItemSubClassId) then
		return PrimarySort(a, b)
	end

	return (aItemSubClassId or 99) < (bItemSubClassId or 99)
end

local function ReverseSort(a, b)
	return DefaultSort(b, a)
end

local function UpdateSorted(source, destination)
	for i, bs in pairs(bagSorted) do
		if bs == source then
			bagSorted[i] = destination
		elseif bs == destination then
			bagSorted[i] = source
		end
	end
end

local function ShouldMove(source, destination)
	if destination == source then return end

	if not bagIDs[source] then return end
	if bagIDs[source] == bagIDs[destination] and bagStacks[source] == bagStacks[destination] then return end

	return true
end

local function IterateForwards(bagList, i)
	i = i + 1
	local step = 1
	for _,bag in ipairs(bagList) do
		local slots = MERB:GetNumSlots(bag, bagRole)
		if i > slots + step then
			step = step + slots
		else
			for slot = 1, slots do
				if step == i then
					return i, bag, slot
				end
				step = step + 1
			end
		end
	end
	bagRole = nil
end

local function IterateBackwards(bagList, i)
	i = i + 1
	local step = 1
	for ii = #bagList, 1, -1 do
		local bag = bagList[ii]
		local slots = MERB:GetNumSlots(bag, bagRole)
		if i > slots + step then
			step = step + slots
		else
			for slot=slots, 1, -1 do
				if step == i then
					return i, bag, slot
				end
				step = step + 1
			end
		end
	end
	bagRole = nil
end

function MERB.IterateBags(bagList, reverse, role)
	bagRole = role
	return (reverse and IterateBackwards or IterateForwards), bagList, 0
end

function MERB:GetItemID(bag, slot)
	if IsGuildBankBag(bag) then
		local link = self:GetItemLink(bag, slot)
		return link and tonumber(string.match(link, "item:(%d+)"))
	else
		return GetContainerItemID(bag, slot)
	end
end

function MERB:GetItemInfo(bag, slot)
	if IsGuildBankBag(bag) then
		return GetGuildBankItemInfo(bag - 50, slot)
	else
		return GetContainerItemInfo(bag, slot)
	end
end

function MERB:GetItemLink(bag, slot)
	if IsGuildBankBag(bag) then
		return GetGuildBankItemLink(bag - 50, slot)
	else
		return GetContainerItemLink(bag, slot)
	end
end

function MERB:PickupItem(bag, slot)
	if IsGuildBankBag(bag) then
		return PickupGuildBankItem(bag - 50, slot)
	else
		return PickupContainerItem(bag, slot)
	end
end

function MERB:SplitItem(bag, slot, amount)
	if IsGuildBankBag(bag) then
		return SplitGuildBankItem(bag - 50, slot, amount)
	else
		return SplitContainerItem(bag, slot, amount)
	end
end

function MERB:GetNumSlots(bag, role)
	if IsGuildBankBag(bag) then
		if not role then role = "deposit" end
		local name, icon, canView, canDeposit, numWithdrawals = GetGuildBankTabInfo(bag - 50)
		if name and canView --[[and ((role == "withdraw" and numWithdrawals ~= 0) or (role == "deposit" and canDeposit) or (role == "both" and numWithdrawals ~= 0 and canDeposit))]] then
			return 98
		end
	else
		return GetContainerNumSlots(bag)
	end

	return 0
end

local function ConvertLinkToID(link)
	if not link then return; end

	if tonumber(string.match(link, "item:(%d+)")) then
		return tonumber(string.match(link, "item:(%d+)"))
	elseif tonumber(string.match(link, "keystone:(%d+)")) then
		return tonumber(string.match(link, "keystone:(%d+)")), nil, true
	else
		return tonumber(string.match(link, "battlepet:(%d+)")), true
	end
end

local function DefaultCanMove()
	return true
end

function MERB:Encode_BagSlot(bag, slot)
	return (bag*100) + slot
end

function MERB:Decode_BagSlot(int)
	return math.floor(int/100), int % 100
end

function MERB:IsPartial(bag, slot)
	local bagSlot = MERB:Encode_BagSlot(bag, slot)
	return ((bagMaxStacks[bagSlot] or 0) - (bagStacks[bagSlot] or 0)) > 0
end

function MERB:EncodeMove(source, target)
	return (source * 10000) + target
end

function MERB:DecodeMove(move)
	local s = math.floor(move/10000)
	local t = move%10000
	s = (t>9000) and (s+1) or s
	t = (t>9000) and (t-10000) or t
	return s, t
end

function MERB:AddMove(source, destination)
	UpdateLocation(source, destination)
	table.insert(moves, 1, MERB:EncodeMove(source, destination))
end

function MERB:ScanBags()
	for _, bag, slot in MERB.IterateBags(allBags) do
		local bagSlot = MERB:Encode_BagSlot(bag, slot)
		local itemID, isBattlePet, isKeystone = ConvertLinkToID(MERB:GetItemLink(bag, slot))
		if itemID then
			if isBattlePet then
				bagPetIDs[bagSlot] = itemID
				bagMaxStacks[bagSlot] = 1
			elseif isKeystone then
				bagMaxStacks[bagSlot] = 1
				bagQualities[bagSlot] = 4
				bagStacks[bagSlot] = 1
			else
				bagMaxStacks[bagSlot] = select(8, GetItemInfo(itemID))
			end

			bagIDs[bagSlot] = itemID
			if not isKeystone then
				bagQualities[bagSlot] = select(3, GetItemInfo(MERB:GetItemLink(bag, slot)))
				bagStacks[bagSlot] = select(2, MERB:GetItemInfo(bag, slot))
			end
		end
	end
end

function MERB:IsSpecialtyBag(bagID)
	if safe[bagID] or IsGuildBankBag(bagID) then return false end

	local inventorySlot = ContainerIDToInventoryID(bagID)
	if not inventorySlot then return false end

	local bag = GetInventoryItemLink("player", inventorySlot)
	if not bag then return false end

	local family = GetItemFamily(bag)
	if family == 0 then return false end

	return family
end

function MERB:CanItemGoInBag(bag, slot, targetBag)
	local item = bagIDs[MERB:Encode_BagSlot(bag, slot)]
	local itemFamily = GetItemFamily(item)
	if itemFamily and itemFamily > 0 then
		local equipSlot = select(9, GetItemInfo(item))
		if equipSlot == "INVTYPE_BAG" then
			itemFamily = 1
		end
	end
	local bagFamily = select(2, GetContainerNumFreeSlots(targetBag))
	if itemFamily then
		return (bagFamily == 0) or bit.band(itemFamily, bagFamily) > 0
	else
		return false
	end
end

function MERB.Compress(...)
	for i=1, select("#", ...) do
		local bags = select(i, ...)
		MERB.Stack(bags, bags, MERB.IsPartial)
	end
end

function MERB.Stack(sourceBags, targetBags, canMove)
	if not canMove then canMove = DefaultCanMove end
	for _, bag, slot in MERB.IterateBags(targetBags, nil, "deposit") do
		local bagSlot = MERB:Encode_BagSlot(bag, slot)
		local itemID = bagIDs[bagSlot]
		if itemID and (bagStacks[bagSlot] ~= bagMaxStacks[bagSlot]) then
			targetItems[itemID] = (targetItems[itemID] or 0) + 1
			table.insert(targetSlots, bagSlot)
		end
	end

	for _, bag, slot in MERB.IterateBags(sourceBags, true, "withdraw") do
		local sourceSlot = MERB:Encode_BagSlot(bag, slot)
		local itemID = bagIDs[sourceSlot]
		if itemID and targetItems[itemID] and canMove(itemID, bag, slot) then
			for i = #targetSlots, 1, -1 do
				local targetedSlot = targetSlots[i]
				if bagIDs[sourceSlot] and bagIDs[targetedSlot] == itemID and targetedSlot ~= sourceSlot and not (bagStacks[targetedSlot] == bagMaxStacks[targetedSlot]) and not sourceUsed[targetedSlot] then
					MERB:AddMove(sourceSlot, targetedSlot)
					sourceUsed[sourceSlot] = true

					if bagStacks[targetedSlot] == bagMaxStacks[targetedSlot] then
						targetItems[itemID] = (targetItems[itemID] > 1) and (targetItems[itemID] - 1) or nil
					end
					if bagStacks[sourceSlot] == 0 then
						targetItems[itemID] = (targetItems[itemID] > 1) and (targetItems[itemID] - 1) or nil
						break
					end
					if not targetItems[itemID] then break end
				end
			end
		end
	end

	table.wipe(targetItems)
	table.wipe(targetSlots)
	table.wipe(sourceUsed)
end

function MERB.Sort(bags, sorter, invertDirection)
	if not sorter then sorter = invertDirection and ReverseSort or DefaultSort end

	for i, bag, slot in MERB.IterateBags(bags, nil, "both") do
		local bagSlot = MERB:Encode_BagSlot(bag, slot)
		initialOrder[bagSlot] = i
		table.insert(bagSorted, bagSlot)
	end

	table.sort(bagSorted, sorter)

	local passNeeded = true
	while passNeeded do
		passNeeded = false
		local i = 1
		for _, bag, slot in MERB.IterateBags(bags, nil, "both") do
			local destination = MERB:Encode_BagSlot(bag, slot)
			local source = bagSorted[i]

			if ShouldMove(source, destination) then
				if not (bagLocked[source] or bagLocked[destination]) then
					MERB:AddMove(source, destination)
					UpdateSorted(source, destination)
					bagLocked[source] = true
					bagLocked[destination] = true

				else
					passNeeded = true
				end
			end
			i = i + 1
		end
		table.wipe(bagLocked)
	end

	table.wipe(bagSorted)
	table.wipe(initialOrder)
end

function MERB.FillBags(from, to)
	MERB.Stack(from, to)
	for _, bag in ipairs(to) do
		if MERB:IsSpecialtyBag(bag) then
			table.insert(specialtyBags, bag)
		end
	end
	if #specialtyBags > 0 then
		MERB:Fill(from, specialtyBags)
	end

	MERB.Fill(from, to)
	table.wipe(specialtyBags)
end

function MERB.Fill(sourceBags, targetBags, reverse, canMove)
	if not canMove then canMove = DefaultCanMove end

	for _, bag, slot in MERB.IterateBags(targetBags, reverse, "deposit") do
		local bagSlot = MERB:Encode_BagSlot(bag, slot)
		if not bagIDs[bagSlot] then
			table.insert(emptySlots, bagSlot)
		end
	end

	for _, bag, slot in MERB.IterateBags(sourceBags, not reverse, "withdraw") do
		if #emptySlots == 0 then break end
		local bagSlot = MERB:Encode_BagSlot(bag, slot)
		local targetBag, targetSlot = MERB:Decode_BagSlot(emptySlots[1])

		local link = MERB:GetItemLink(bag, slot);

		if bagIDs[bagSlot] and MERB:CanItemGoInBag(bag, slot, targetBag) and canMove(bagIDs[bagSlot], bag, slot) then
			MERB:AddMove(bagSlot, tremove(emptySlots, 1))
		end
	end
	table.wipe(emptySlots)
end

function MERB.SortBags(...)
	for i=1, select("#", ...) do
		local bags = select(i, ...)
		for _, slotNum in ipairs(bags) do
			local bagType = MERB:IsSpecialtyBag(slotNum)
			if bagType == false then bagType = "Normal" end
			if not bagCache[bagType] then bagCache[bagType] = {} end
			table.insert(bagCache[bagType], slotNum)
		end

		for bagType, sortedBags in pairs(bagCache) do
			if bagType ~= "Normal" then
				MERB.Stack(sortedBags, sortedBags, MERB.IsPartial)
				MERB.Stack(bagCache["Normal"], sortedBags)
				MERB.Fill(bagCache["Normal"], sortedBags, E.db.mui.bags.sortInverted)
				MERB.Sort(sortedBags, nil, E.db.mui.bags.sortInverted)
				table.wipe(sortedBags)
			end
		end

		if bagCache["Normal"] then
			MERB.Stack(bagCache["Normal"], bagCache["Normal"], MERB.IsPartial)
			MERB.Sort(bagCache["Normal"], nil, E.db.mui.bags.sortInverted)
			table.wipe(bagCache["Normal"])
		end
		table.wipe(bagCache)
		table.wipe(bagGroups)
	end
end

function MERB:StartStacking()
	table.wipe(bagMaxStacks)
	table.wipe(bagStacks)
	table.wipe(bagIDs)
	table.wipe(bagQualities)
	table.wipe(bagPetIDs)
	table.wipe(moveTracker)

	if #moves > 0 then
		self.SortUpdateTimer:Show()
	else
		MERB:StopStacking()
	end
end

function MERB:StopStacking(message)
	table.wipe(moves)
	table.wipe(moveTracker)
	moveRetries, lastItemID, lockStop, lastDestination, lastMove = 0, nil, nil, nil, nil
	self.SortUpdateTimer:Hide()
	if message then
		MER:Print(message)
	end
end

function MERB:DoMove(move)
	if GetCursorInfo() == "item" then
		return false, 'cursorhasitem'
	end

	local source, target = MERB:DecodeMove(move)
	local sourceBag, sourceSlot = MERB:Decode_BagSlot(source)
	local targetBag, targetSlot = MERB:Decode_BagSlot(target)
	local _, sourceCount, sourceLocked = MERB:GetItemInfo(sourceBag, sourceSlot)
	local _, targetCount, targetLocked = MERB:GetItemInfo(targetBag, targetSlot)

	if sourceLocked or targetLocked then
		return false, "source/target_locked"
	end

	local sourceLink = MERB:GetItemLink(sourceBag, sourceSlot)
	local sourceItemID = self:GetItemID(sourceBag, sourceSlot)
	local targetItemID = self:GetItemID(targetBag, targetSlot)
	if not sourceItemID then
		if moveTracker[source] then
			return false, "move incomplete"
		else
			return MERB:StopStacking(L["Some error happened, please try again."])
		end
	end

	local stackSize = select(8, GetItemInfo(sourceItemID))
	if (sourceItemID == targetItemID) and (targetCount ~= stackSize) and ((targetCount + sourceCount) > stackSize) then
		MERB:SplitItem(sourceBag, sourceSlot, stackSize - targetCount)
	else
		MERB:PickupItem(sourceBag, sourceSlot)
	end

	if GetCursorInfo() == "item" then
		MERB:PickupItem(targetBag, targetSlot)
	end

	local sourceGuild = IsGuildBankBag(sourceBag)
	local targetGuild = IsGuildBankBag(targetBag)

	if sourceGuild then
		QueryGuildBankTab(sourceBag - 50)
	end
	if targetGuild then
		QueryGuildBankTab(targetBag - 50)
	end
	return true, sourceItemID, source, targetItemID, target
end

function MERB:DoMoves()
	if InCombatLockdown() then
		return MERB:StopStacking(L["ome error happened, please try again."])
	end

	local cursorType, cursorItemID = GetCursorInfo()
	if cursorType == "item" and cursorItemID then
		if lastItemID ~= cursorItemID then
			return MERB:StopStacking(L["ome error happened, please try again."])
		end

		if moveRetries < 100 then
			local targetBag, targetSlot = self:Decode_BagSlot(lastDestination)
			local _, _, targetLocked = self:GetItemInfo(targetBag, targetSlot)
			if not targetLocked then
				self:PickupItem(targetBag, targetSlot)
				WAIT_TIME = 0.1
				lockStop = GetTime()
				moveRetries = moveRetries + 1
				return
			end
		end
	end

	if lockStop then
		for slot, itemID in pairs(moveTracker) do
			local actualItemID = self:GetItemID(self:Decode_BagSlot(slot))
			if actualItemID ~= itemID then
				WAIT_TIME = 0.1
				if (GetTime() - lockStop) > MAX_MOVE_TIME then
					if lastMove and moveRetries < 100 then
						local success, moveID, moveSource, targetID, moveTarget, wasGuild = self:DoMove(lastMove)
						WAIT_TIME = wasGuild and 0.5 or 0.1

						if not success then
							lockStop = GetTime()
							moveRetries = moveRetries + 1
							return
						end

						moveTracker[moveSource] = targetID
						moveTracker[moveTarget] = moveID
						lastDestination = moveTarget
						lastItemID = moveID
						return
					end

					MERB:StopStacking()
					return
				end
				return --give processing time to happen
			end
			moveTracker[slot] = nil
		end
	end

	lastItemID, lockStop, lastDestination, lastMove = nil, nil, nil, nil
	table.wipe(moveTracker)

	local start, success, moveID, targetID, moveSource, moveTarget, wasGuild
	start = GetTime()
	if #moves > 0 then
		for i = #moves, 1, -1 do
			success, moveID, moveSource, targetID, moveTarget, wasGuild = MERB:DoMove(moves[i])
			if not success then
				WAIT_TIME = wasGuild and 0.3 or 0.1
				lockStop = GetTime()
				return
			end
			moveTracker[moveSource] = targetID
			moveTracker[moveTarget] = moveID
			lastDestination = moveTarget
			lastMove = moves[i]
			lastItemID = moveID
			table.remove(moves, i)
			if moves[i-1] then
				WAIT_TIME = wasGuild and 0.3 or 0;
				return
			end
		end
	end
	MERB:StopStacking()
end

function MERB:GetGroup(id)
	if string.match(id, "^[-%d,]+$") then
		local bags = {}
		for b in string.gmatch(id, "-?%d+") do
			table.insert(bags, tonumber(b))
		end
		return bags
	end
	return coreGroups[id]
end

function MERB:CommandDecorator(func, groupsDefaults)
	local bagGroups = {}
	return function(groups)
		if self.SortUpdateTimer:IsShown() then
			MER:Print(L["Already working"])
			MERBB:StopStacking()
			return
		end

		table.wipe(bagGroups)
		if not groups or #groups == 0 then
			groups = groupsDefaults
		end
		for bags in (groups or ""):gmatch("[^%s]+") do
			if bags == "guild" then
				bags = MERB:GetGroup(bags)
				if bags then
					tinsert(bagGroups, {bags[GetCurrentGuildBankTab()]})
				end
			else
				bags = MERB:GetGroup(bags)
				if bags then
					tinsert(bagGroups, bags)
				end
			end
		end

		MERB:ScanBags()
		if func(unpack(bagGroups)) == false then
			return
		end
		table.wipe(bagGroups)
		MERB:StartStacking()
	end
end